/* Hardware-dependent portions of the WTX protocol
   used with a VxWorks target

   Copyright 2003, 2004 Free Software Foundation, Inc.

   This file is part of GDB.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include "defs.h"
#include "remote-wtx.h"

/* number of registers of the target.  */

#ifdef VXWORKS_55_TARGET
#define WTXPPC_REGISTER_NUMBER 153
#else
#define WTXPPC_REGISTER_NUMBER 119
#endif
#define WTXARM_REGISTER_NUMBER 26
#define WTXSP64_REGISTER_NUMBER 126

#define REG_ERROR -1

const int              invalid_register_number = REG_ERROR;
const UINT32           invalid_register_offset = REG_ERROR;
const UINT32           invalid_register_size   = REG_ERROR;
const WTX_REG_SET_TYPE invalid_register_set    = REG_ERROR;

/* REGISTER_TABLE associates a GDB register number with its WTX
   representation.  */

static const struct wtx_register
wtxppc_register_table [WTXPPC_REGISTER_NUMBER] =
  {
    /* General purpose registers.  */

    {0x00, 4, WTX_REG_SET_IU},   /* r0     (0) */
    {0x04, 4, WTX_REG_SET_IU},   /* r1 */
    {0x08, 4, WTX_REG_SET_IU},   /* r2 */
    {0x0c, 4, WTX_REG_SET_IU},   /* r3 */
    {0x10, 4, WTX_REG_SET_IU},   /* r4 */
    {0x14, 4, WTX_REG_SET_IU},   /* r5 */
    {0x18, 4, WTX_REG_SET_IU},   /* r6 */
    {0x1c, 4, WTX_REG_SET_IU},   /* r7 */
    {0x20, 4, WTX_REG_SET_IU},   /* r8     (8) */
    {0x24, 4, WTX_REG_SET_IU},   /* r9 */
    {0x28, 4, WTX_REG_SET_IU},   /* r10 */
    {0x2c, 4, WTX_REG_SET_IU},   /* r11 */
    {0x30, 4, WTX_REG_SET_IU},   /* r12 */
    {0x34, 4, WTX_REG_SET_IU},   /* r13 */
    {0x38, 4, WTX_REG_SET_IU},   /* r14 */
    {0x3c, 4, WTX_REG_SET_IU},   /* r15    (16) */
    {0x40, 4, WTX_REG_SET_IU},   /* r16 */
    {0x44, 4, WTX_REG_SET_IU},   /* r17 */
    {0x48, 4, WTX_REG_SET_IU},   /* r18 */
    {0x4c, 4, WTX_REG_SET_IU},   /* r19 */
    {0x50, 4, WTX_REG_SET_IU},   /* r20 */
    {0x54, 4, WTX_REG_SET_IU},   /* r21 */
    {0x58, 4, WTX_REG_SET_IU},   /* r22 */
    {0x5c, 4, WTX_REG_SET_IU},   /* r23 */
    {0x60, 4, WTX_REG_SET_IU},   /* r24    (24) */
    {0x64, 4, WTX_REG_SET_IU},   /* r25 */
    {0x68, 4, WTX_REG_SET_IU},   /* r26 */
    {0x6c, 4, WTX_REG_SET_IU},   /* r27 */
    {0x70, 4, WTX_REG_SET_IU},   /* r28 */
    {0x74, 4, WTX_REG_SET_IU},   /* r29 */
    {0x78, 4, WTX_REG_SET_IU},   /* r30 */
    {0x7c, 4, WTX_REG_SET_IU},   /* r31 */
    {0x00, 8, WTX_REG_SET_FPU},  /* f0     (32) */
    {0x08, 8, WTX_REG_SET_FPU},  /* f1 */
    {0x10, 8, WTX_REG_SET_FPU},  /* f2 */
    {0x18, 8, WTX_REG_SET_FPU},  /* f3 */
    {0x20, 8, WTX_REG_SET_FPU},  /* f4 */
    {0x28, 8, WTX_REG_SET_FPU},  /* f5 */
    {0x30, 8, WTX_REG_SET_FPU},  /* f6 */
    {0x38, 8, WTX_REG_SET_FPU},  /* f7 */
    {0x40, 8, WTX_REG_SET_FPU},  /* f8     (40) */
    {0x48, 8, WTX_REG_SET_FPU},  /* f9 */
    {0x50, 8, WTX_REG_SET_FPU},  /* f10 */
    {0x58, 8, WTX_REG_SET_FPU},  /* f11 */
    {0x60, 8, WTX_REG_SET_FPU},  /* f12 */
    {0x68, 8, WTX_REG_SET_FPU},  /* f13 */
    {0x70, 8, WTX_REG_SET_FPU},  /* f14 */
    {0x78, 8, WTX_REG_SET_FPU},  /* f15 */
    {0x80, 8, WTX_REG_SET_FPU},  /* f16    (48) */
    {0x88, 8, WTX_REG_SET_FPU},  /* f17 */
    {0x90, 8, WTX_REG_SET_FPU},  /* f18 */
    {0x98, 8, WTX_REG_SET_FPU},  /* f19 */
    {0xa0, 8, WTX_REG_SET_FPU},  /* f20 */
    {0xa8, 8, WTX_REG_SET_FPU},  /* f21 */
    {0xb0, 8, WTX_REG_SET_FPU},  /* f22 */
    {0xb8, 8, WTX_REG_SET_FPU},  /* f23 */
    {0xc0, 8, WTX_REG_SET_FPU},  /* f24    (56) */
    {0xc8, 8, WTX_REG_SET_FPU},  /* f25 */
    {0xd0, 8, WTX_REG_SET_FPU},  /* f26 */
    {0xd8, 8, WTX_REG_SET_FPU},  /* f27 */
    {0xe0, 8, WTX_REG_SET_FPU},  /* f28 */
    {0xe8, 8, WTX_REG_SET_FPU},  /* f29 */
    {0xf0, 8, WTX_REG_SET_FPU},  /* f30 */
    {0xf8, 8, WTX_REG_SET_FPU},  /* f31 */
    {0x8c, 4, WTX_REG_SET_IU},   /* pc     (64) */
    {0x80, 4, WTX_REG_SET_IU},   /* ps/msr */

    /* Special purpose registers.  */

    {0x90,  4, WTX_REG_SET_IU},   /* cr     (66) */
    {0x84,  4, WTX_REG_SET_IU},   /* lr */
    {0x88,  4, WTX_REG_SET_IU},   /* ctr */
    {0x94,  4, WTX_REG_SET_IU},   /* xer */
    {0x100, 4, WTX_REG_SET_FPU}, /* fpscr */

    /* Segment registers (not accessible).  */

    {0x98,       4,         WTX_REG_SET_IU},   /* mq/sr0 (71) */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* sr1 */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* sr2 */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* sr3 */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* sr4 */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* sr5 */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* sr6 */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* sr7 */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* sr8    (79) */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* sr9  */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* sr10 */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* sr11 */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* sr12 */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* sr13 */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* sr14 */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* sr15 */

    /* OEA SPRs (not accessible).  */

    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* pvr    */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* ibat0u (88) */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* ibat0l */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* ibat1u */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* ibat1l */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* ibat2u */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* ibat2l */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* ibat3u */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* ibat3l */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* dbat0u (96) */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* dbat0l */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* dbat1u */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* dbat1l */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* dbat2u */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* dbat2l */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* dbat3u */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* dbat3l */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* sdr1   (104) */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* asr */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* dar */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* dsisr */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* sprg0 */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* sprg1 */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* sprg2 */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* sprg3 */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* srr0   (112) */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* srr1 */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* tbl */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* tbu */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* dec */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* dabr */
    {REG_ERROR,  REG_ERROR, WTX_REG_SET_IU},   /* ear */

    /* AltiVec registers. Only supported since VxWorks 5.5.  */

#ifdef VXWORKS_55_TARGET
    {0x00,  16, WTX_REG_SET_AV},  /* vr0    (119) */
    {0x10,  16, WTX_REG_SET_AV},  /* vr1 */
    {0x20,  16, WTX_REG_SET_AV},  /* vr2 */
    {0x30,  16, WTX_REG_SET_AV},  /* vr3 */
    {0x40,  16, WTX_REG_SET_AV},  /* vr4 */
    {0x50,  16, WTX_REG_SET_AV},  /* vr5 */
    {0x60,  16, WTX_REG_SET_AV},  /* vr6 */
    {0x70,  16, WTX_REG_SET_AV},  /* vr7 */
    {0x80,  16, WTX_REG_SET_AV},  /* vr8    (127) */
    {0x90,  16, WTX_REG_SET_AV},  /* vr9 */
    {0xa0,  16, WTX_REG_SET_AV},  /* vr10 */
    {0xb0,  16, WTX_REG_SET_AV},  /* vr11 */
    {0xc0,  16, WTX_REG_SET_AV},  /* vr12 */
    {0xd0,  16, WTX_REG_SET_AV},  /* vr13 */
    {0xe0,  16, WTX_REG_SET_AV},  /* vr14 */
    {0xf0,  16, WTX_REG_SET_AV},  /* vr15 */
    {0x100, 16, WTX_REG_SET_AV}, /* vr16   (135) */
    {0x110, 16, WTX_REG_SET_AV}, /* vr17 */
    {0x120, 16, WTX_REG_SET_AV}, /* vr18 */
    {0x130, 16, WTX_REG_SET_AV}, /* vr19 */
    {0x140, 16, WTX_REG_SET_AV}, /* vr20 */
    {0x150, 16, WTX_REG_SET_AV}, /* vr21 */
    {0x160, 16, WTX_REG_SET_AV}, /* vr22 */
    {0x170, 16, WTX_REG_SET_AV}, /* vr23 */
    {0x180, 16, WTX_REG_SET_AV}, /* vr24    (143) */
    {0x190, 16, WTX_REG_SET_AV}, /* vr25 */
    {0x1a0, 16, WTX_REG_SET_AV}, /* vr26 */
    {0x1b0, 16, WTX_REG_SET_AV}, /* vr27 */
    {0x1c0, 16, WTX_REG_SET_AV}, /* vr28 */
    {0x1d0, 16, WTX_REG_SET_AV}, /* vr29 */
    {0x1e0, 16, WTX_REG_SET_AV}, /* vr30 */
    {0x1f0, 16, WTX_REG_SET_AV}, /* vr31 */
    {0x200, 4,  WTX_REG_SET_AV},  /* vscr    (151) */
    {0x210, 4,  WTX_REG_SET_AV}  /* vrsave */
#endif
  };

/* REGISTER_TABLE associates a GDB register number with its WTX
   representation.  */

static const struct wtx_register
wtxarm_register_table [WTXARM_REGISTER_NUMBER] =
  {
    /* General purpose registers.  */

    {0x00, 4, WTX_REG_SET_IU}, /* r0  (0) */
    {0x04, 4, WTX_REG_SET_IU}, /* r1 */
    {0x08, 4, WTX_REG_SET_IU}, /* r2 */
    {0x0c, 4, WTX_REG_SET_IU}, /* r3 */
    {0x10, 4, WTX_REG_SET_IU}, /* r4  (4)*/
    {0x14, 4, WTX_REG_SET_IU}, /* r5 */
    {0x18, 4, WTX_REG_SET_IU}, /* r6 */
    {0x1c, 4, WTX_REG_SET_IU}, /* r7 */
    {0x20, 4, WTX_REG_SET_IU}, /* r8  (8) */
    {0x24, 4, WTX_REG_SET_IU}, /* r9 */
    {0x28, 4, WTX_REG_SET_IU}, /* r10 */
    {0x2c, 4, WTX_REG_SET_IU}, /* r11 */
    {0x30, 4, WTX_REG_SET_IU}, /* ip  (12) */
    {0x34, 4, WTX_REG_SET_IU}, /* sp */
    {0x38, 4, WTX_REG_SET_IU}, /* lr */
    {0x3c, 4, WTX_REG_SET_IU}, /* pc */

    /* Floating point registers (not accessible).  */

    {REG_ERROR, REG_ERROR, WTX_REG_SET_FPU}, /* f0  (16)*/
    {REG_ERROR, REG_ERROR, WTX_REG_SET_FPU}, /* f1 */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_FPU}, /* f2 */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_FPU}, /* f3 */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_FPU}, /* f4  (20) */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_FPU}, /* f5 */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_FPU}, /* f6 */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_FPU}, /* f7 */
    
    /* Special purpose registers.  */

    {REG_ERROR, REG_ERROR, WTX_REG_SET_FPU}, /* fps (24) */
    {0x40,      4,         WTX_REG_SET_IU}, /* cpsr */
  };

static const struct wtx_register
wtxsp64_register_table [WTXSP64_REGISTER_NUMBER] =
  {
    {0x28,      8,         WTX_REG_SET_IU},  /* g0          (0) */
    {0x30,      8,         WTX_REG_SET_IU},  /* g1 */
    {0x38,      8,         WTX_REG_SET_IU},  /* g2 */
    {0x40,      8,         WTX_REG_SET_IU},  /* g3 */
    {0x48,      8,         WTX_REG_SET_IU},  /* g4 */
    {0x50,      8,         WTX_REG_SET_IU},  /* g5 */
    {0x58,      8,         WTX_REG_SET_IU},  /* g6 */
    {0x60,      8,         WTX_REG_SET_IU},  /* g7 */
    {0x68,      8,         WTX_REG_SET_IU},  /* o0          (8)*/
    {0x70,      8,         WTX_REG_SET_IU},  /* o1 */
    {0x78,      8,         WTX_REG_SET_IU},  /* o2 */
    {0x80,      8,         WTX_REG_SET_IU},  /* o3 */
    {0x88,      8,         WTX_REG_SET_IU},  /* o4 */
    {0x90,      8,         WTX_REG_SET_IU},  /* o5 */
    {0x98,      8,         WTX_REG_SET_IU},  /* sp */
    {0xa0,      8,         WTX_REG_SET_IU},  /* o7 */
    {0xa8,      8,         WTX_REG_SET_IU},  /* l0          (16) */
    {0xb0,      8,         WTX_REG_SET_IU},  /* l1 */
    {0xb8,      8,         WTX_REG_SET_IU},  /* l2 */
    {0xc0,      8,         WTX_REG_SET_IU},  /* l3 */
    {0xc8,      8,         WTX_REG_SET_IU},  /* l4 */
    {0xd0,      8,         WTX_REG_SET_IU},  /* l5 */
    {0xd8,      8,         WTX_REG_SET_IU},  /* l6 */
    {0xe0,      8,         WTX_REG_SET_IU},  /* l7 */
    {0xe8,      8,         WTX_REG_SET_IU},  /* i0          (24) */
    {0xf0,      8,         WTX_REG_SET_IU},  /* i1 */
    {0xf8,      8,         WTX_REG_SET_IU},  /* i2 */
    {0x100,     8,         WTX_REG_SET_IU}, /* i3 */
    {0x108,     8,         WTX_REG_SET_IU}, /* i4 */
    {0x110,     8,         WTX_REG_SET_IU}, /* i5 */
    {0x118,     8,         WTX_REG_SET_IU}, /* i6/fp */
    {0x120,     8,         WTX_REG_SET_IU}, /* i7 */
    {0x00,      4,         WTX_REG_SET_FPU}, /* f0          (32) */
    {0x04,      4,         WTX_REG_SET_FPU}, /* f1 */
    {0x08,      4,         WTX_REG_SET_FPU}, /* f2 */
    {0x0c,      4,         WTX_REG_SET_FPU}, /* f3 */
    {0x10,      4,         WTX_REG_SET_FPU}, /* f4 */
    {0x14,      4,         WTX_REG_SET_FPU}, /* f5 */
    {0x18,      4,         WTX_REG_SET_FPU}, /* f6 */
    {0x1c,      4,         WTX_REG_SET_FPU}, /* f7 */
    {0x20,      4,         WTX_REG_SET_FPU}, /* f8          (40) */
    {0x24,      4,         WTX_REG_SET_FPU}, /* f9  */
    {0x28,      4,         WTX_REG_SET_FPU}, /* f10 */
    {0x2c,      4,         WTX_REG_SET_FPU}, /* f11 */
    {0x30,      4,         WTX_REG_SET_FPU}, /* f12 */
    {0x34,      4,         WTX_REG_SET_FPU}, /* f13 */
    {0x38,      4,         WTX_REG_SET_FPU}, /* f14 */
    {0x3c,      4,         WTX_REG_SET_FPU}, /* f15 */
    {0x40,      4,         WTX_REG_SET_FPU}, /* f16         (48)*/
    {0x44,      4,         WTX_REG_SET_FPU}, /* f17 */
    {0x48,      4,         WTX_REG_SET_FPU}, /* f18 */
    {0x4c,      4,         WTX_REG_SET_FPU}, /* f19 */
    {0x50,      4,         WTX_REG_SET_FPU}, /* f20 */
    {0x54,      4,         WTX_REG_SET_FPU}, /* f21 */
    {0x58,      4,         WTX_REG_SET_FPU}, /* f22 */
    {0x5c,      4,         WTX_REG_SET_FPU}, /* f23 */
    {0x60,      4,         WTX_REG_SET_FPU}, /* f24         (56) */
    {0x64,      4,         WTX_REG_SET_FPU}, /* f25 */
    {0x68,      4,         WTX_REG_SET_FPU}, /* f26 */
    {0x6c,      4,         WTX_REG_SET_FPU}, /* f27 */
    {0x70,      4,         WTX_REG_SET_FPU}, /* f28 */
    {0x74,      4,         WTX_REG_SET_FPU}, /* f29 */
    {0x78,      4,         WTX_REG_SET_FPU}, /* f30 */
    {0x7c,      4,         WTX_REG_SET_FPU}, /* f31 */
    {0x80,      8,         WTX_REG_SET_FPU}, /* f32         (64) */
    {0x88,      8,         WTX_REG_SET_FPU}, /* f34 */
    {0x90,      8,         WTX_REG_SET_FPU}, /* f36 */
    {0x98,      8,         WTX_REG_SET_FPU}, /* f38 */
    {0xa0,      8,         WTX_REG_SET_FPU}, /* f40 */
    {0xa8,      8,         WTX_REG_SET_FPU}, /* f42 */
    {0xb0,      8,         WTX_REG_SET_FPU}, /* f44 */
    {0xb8,      8,         WTX_REG_SET_FPU}, /* f46 */
    {0xc0,      8,         WTX_REG_SET_FPU}, /* f48         (72) */
    {0xc8,      8,         WTX_REG_SET_FPU}, /* f50 */
    {0xd0,      8,         WTX_REG_SET_FPU}, /* f52 */
    {0xd8,      8,         WTX_REG_SET_FPU}, /* f54 */
    {0xe0,      8,         WTX_REG_SET_FPU},  /* f56 */
    {0xe8,      8,         WTX_REG_SET_FPU},  /* f58 */
    {0xf0,      8,         WTX_REG_SET_FPU},  /* f60 */
    {0xf8,      8,         WTX_REG_SET_FPU},  /* f62 */
    {0x00,      4,         WTX_REG_SET_IU},   /* pc         (80) */
    {0x04,      4,         WTX_REG_SET_IU},   /* npc */
    {0x0b,      1,         WTX_REG_SET_IU},   /* ccr */
    {0x104,     4,         WTX_REG_SET_FPU}, /* fsr */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* fprs */
    {0x1c,      4,         WTX_REG_SET_IU},   /* y */
    {0x0c,      1,         WTX_REG_SET_IU},   /* asi */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* ver */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* tick       (88) */
    {0x18,      4,         WTX_REG_SET_IU},   /* pil */
    {0x0d,      2,         WTX_REG_SET_IU},   /* pstate */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* tstate */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* tba */
    {0x14,      4,         WTX_REG_SET_IU},   /* tl */
    {0x10,      4,         WTX_REG_SET_IU},   /* tt */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* tpc */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* tnpc       (96) */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* wstate */
    {0x0f,      1,         WTX_REG_SET_IU},   /* cwp */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* cansave */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* canrestore */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* cleanwin */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* otherwin */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* asr16      (104) */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* asr17 */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* asr18 */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* asr19 */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* asr20 */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* asr21 */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* asr22 */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* asr23 */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* asr24      (112) */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* asr25 */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* asr26 */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* asr27 */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* asr28 */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* asr29 */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* asr30 */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* asr31 */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* icc        (120) */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* xcc */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* fcc0 */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* fcc1 */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* fcc2 */
    {REG_ERROR, REG_ERROR, WTX_REG_SET_IU},    /* fcc3 */
  };

/* Target Ops */

struct wtx_target_ops wtxppc_ops;
struct wtx_target_ops wtxarm_ops;
struct wtx_target_ops wtxsp64_ops;

void
_initialize_remote_wtx_hw (void)
{
  wtxppc_ops.to_register_number = WTXPPC_REGISTER_NUMBER;
  wtxppc_ops.to_register_table = wtxppc_register_table;
  wtxarm_ops.to_register_number = WTXARM_REGISTER_NUMBER;
  wtxarm_ops.to_register_table = wtxarm_register_table;
  wtxsp64_ops.to_register_number = WTXPPC_REGISTER_NUMBER;
  wtxsp64_ops.to_register_table = wtxsp64_register_table;

  register_wtx_target_ops (bfd_arch_powerpc, &wtxppc_ops);
  register_wtx_target_ops (bfd_arch_rs6000, &wtxppc_ops);
  register_wtx_target_ops (bfd_arch_arm, &wtxppc_ops);
  register_wtx_osabi_ops (bfd_arch_sparc, bfd_mach_sparc_v9, &wtxsp64_ops);
  register_wtx_osabi_ops (bfd_arch_sparc, bfd_mach_sparc_v9a, &wtxsp64_ops);
}
