/* remote-wtx.h - remote backend for gdb using WTX protocol

   Copyright 2003, 2004 Free Software Foundation, Inc.

   This file is part of GDB.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include "bfd.h"

/* For wtx.h : GDB is run on the host.  */
#define HOST

#include "wtx.h"

/* WTX representation of a target register, different from GDB's one.
   The WTX API provides only operations on register blocks. To be able
   to manipulate one particular register, one should know in which
   register set it can be found, and where this register is the
   corresponding register block.  */

struct wtx_register {
  /* Offset from the beginning of the block.  */
  UINT32 register_offset;

  /* Size of the register, in bytes.  */
  UINT32 register_size;

  /* Register set that contains this register.  */
  WTX_REG_SET_TYPE register_set;
};

/* Invalid values, used when the WTX API cannot access the register.  */

extern const int invalid_register_number;
extern const UINT32 invalid_register_offset;
extern const UINT32 invalid_register_size;
extern const WTX_REG_SET_TYPE invalid_register_set;

/* Target specific attributes and operations.  */

struct wtx_target_ops
{
  /* Number of registers (WTX view).  */
  int to_register_number;

  /* Register table.  */
  const struct wtx_register* to_register_table;
};

/* Register a wtx_ops for a given target architecture.  */

extern void register_wtx_target_ops (enum bfd_architecture arch,
				     struct wtx_target_ops* ops);

/* Register a wtx_ops for a given target architecture variant.  */

extern void register_wtx_osabi_ops (enum bfd_architecture arch,
				    long unsigned int mach,
				    struct wtx_target_ops* ops);
